package org.greenstone.gsdl3.action;

import org.greenstone.gsdl3.core.ModuleInterface;
import org.greenstone.gsdl3.util.*;
// XML classes
import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;
import java.io.File;

public class BuildAction extends Action {
    

    public Element process (Element message) {

	//find the stylesheet - cant go any further if not found
	String stylesheet = GSFile.stylesheetFile(config_, "build.xsl");

	if (stylesheet==null) {
	    System.err.println("BuildAction Error: build stylesheet not found!");
	    return null;
	}

	Element request = (Element)message.getElementsByTagName(GSXML.REQUEST_ELEM).item(0);
	String service_name = request.getAttribute(GSXML.SUBACTION_ATT); 
	// get the param list
	Element cgi_param_list = (Element)GSXML.getChildByTagName(request, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	HashMap params = GSXML.extractParams(cgi_param_list);

	// create the return page tree
	Element page = doc_.createElement(GSXML.PAGE_ELEM);
	page.setAttribute(GSXML.LANG_ATT, request.getAttribute(GSXML.LANG_ATT));
	// add the lang stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.DISPLAY_ELEM), true));
	// add the system stuff from message
	page.appendChild(doc_.importNode(GSXML.getChildByTagName(message, GSXML.CONFIGURATION_ELEM), true));
	

	//  we need to do the request before getting the description- 
	// it may have changedthe action may change the descripiton, eg 
	// deleting a collection changes the collection list
	String collection = (String)params.get("collection");
	String to = "build"; // knwo this cos its build action - loaded by a cluster called build
	to = GSPath.appendLink(to, service_name);
	
	if (collection != null && !collection.equals("")) {
	    
	    // a collection has been specified - do the request
	    Element mr_query_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	    Element mr_query_request = doc_.createElement(GSXML.REQUEST_ELEM);
	    mr_query_message.appendChild(mr_query_request);
	    
	    mr_query_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_BUILD);
	    mr_query_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));
	    
	    mr_query_request.setAttribute(GSXML.TO_ATT, to);
	    
	    // paramList
	    Element query_param_list = (Element) doc_.importNode(cgi_param_list, true);
	    mr_query_request.appendChild(query_param_list);
	    
	    Element mr_query_response = (Element)mr_.process(mr_query_message);
	    Element result_response = (Element)GSXML.getChildByTagName(mr_query_response, GSXML.RESPONSE_ELEM);
	    // append the response to the page
	    page.appendChild(doc_.importNode(result_response, true));
	}

	// another part of the page is the service description

	// request the service info for the selected service - should be cached
	Element mr_info_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element mr_info_request = doc_.createElement(GSXML.REQUEST_ELEM);
	mr_info_message.appendChild(mr_info_request);
	mr_info_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_DESCRIBE);
	mr_info_request.setAttribute(GSXML.LANG_ATT, page.getAttribute(GSXML.LANG_ATT));

	mr_info_request.setAttribute(GSXML.TO_ATT, to);
	
	Element mr_info_response = (Element) mr_.process(mr_info_message);

	String path = GSXML.RESPONSE_ELEM;
	path = GSPath.appendLink(path, GSXML.SERVICE_ELEM);
	Element description = (Element)doc_.importNode(GSXML.getNodeByPath(mr_info_response, path), true);

	Element pl = (Element)GSXML.getChildByTagName(description, GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	
	if (pl !=null) {
	    // add short names to the params in the param list
	    cgi_.addShortNames(pl);

	    // for each param in the description, overwrite teh default value with the currently set value if present 
	    Element param = (Element)pl.getFirstChild();
	    while (param !=null) {
		if (param.getNodeName().equals(GSXML.PARAM_ELEM)) { // just in case
		    String name = param.getAttribute(GSXML.NAME_ATT);
		    String current = (String)params.get(name);
		    if (current !=null && !current.equals("")) {
			param.setAttribute(GSXML.DEFAULT_ATT, current);
		    }
		}
		param = (Element)param.getNextSibling();
	    }
	}
	page.appendChild(description);

  	// part of the data for the page is the cgi-params
	// if we have this here, do we need to do the previous step?
	Element cgi_request = (Element)doc_.importNode(request, true);
	page.appendChild(cgi_request);
	

	// now process the page and return the result
	Document style_doc = converter_.getDOM(new File(stylesheet));
	
	GSXSLT.absoluteIncludePaths(style_doc, config_);
	return (Element)transformer_.transform(style_doc, page);    
    }     
}
